@@ -1,4 +1,3 @@
-import re
 from django.db import models
 from django.db.models.query import QuerySet
 from django.core.urlresolvers import reverse
@@ -9,6 +8,7 @@ from model_utils import Choices
 from model_utils.managers import PassThroughManager
 from model_utils.fields import StatusField
 
+from sorl.thumbnail import ImageField
 from tinymce import models as tinymce_models
 
 
@@ -21,7 +21,6 @@ class SiteEntity(models.Model):
     STATUS = Choices(
         ('draft', 'Draft'),
         ('live', 'Live'),
-        ('deleted', 'Deleted')
     )
 
     title = models.CharField(max_length=255)
@@ -31,41 +30,32 @@ class SiteEntity(models.Model):
 
     objects = PassThroughManager.for_queryset_class(SiteEntityQuerySet)()
 
+    def __str__(self):
+        return self.title
+
     def __unicode__(self):
         return self.title
 
+    class Meta:
+        ordering = ['order', 'title']
+
 
 class Displayable(SiteEntity):
     body = tinymce_models.HTMLField()
 
 
-class Section(SiteEntity):
-    url_override = models.CharField(max_length=255, null=True, blank=True)
-
-    objects = PassThroughManager.for_queryset_class(SiteEntityQuerySet)()
-
-    @property
-    def url(self):
-        if self.url_override:
-            if re.match(r'^(?:https?://)?(?:[\w]+\.)(?:\.?[\w]{2,})+$',
-                self.url_override):
-                return self.url_override
-            else:
-                # Todo: Make this more reliable for generating navigation based
-                #       on sections.
-                return '/' + self.url_override
-        try:
-            return self.pages.live()[0].url
-        except IndexError:
-            return "#"
-
-    class Meta:
-        ordering = ['order']
+class Attachment(SiteEntity):
+    TYPE = Choices(
+        ('image', 'Image'),
+        ('video', 'Video'),
+        ('misc', 'Miscellaneous')
+    )
+    type = models.CharField(max_length=45, choices=TYPE, default=TYPE.image)
+    image = ImageField(upload_to='uploads', null=True, blank=True)
+    file = models.FileField(upload_to='uploads', null=True, blank=True)
 
 
 class Page(MPTTModel, Displayable):
-    section = models.ForeignKey(Section, related_name='pages')
-
     template_override = models.CharField(max_length=255, null=True,
         blank=True)
 
@@ -76,17 +66,38 @@ class Page(MPTTModel, Displayable):
 
     @property
     def url(self):
-        kwargs = {}
-        require_section_slug = False
-        if self != self.section.pages.live()[0]:
-            kwargs['page_slug'] = self.slug
-            require_section_slug = True
+        if self.is_root_node():
+            return reverse('suave:page')
 
-        if self.section != Section.objects.live()[0] or \
-            require_section_slug:
-            kwargs['section_slug'] = self.section.slug
+        crumbs = []
+        for ancestor in self.get_ancestors():
+            if ancestor.is_root_node():
+                continue
+            crumbs.append(ancestor.slug)
+        crumbs.append(self.slug)
+        return reverse('suave:page', kwargs=dict(
+            url='/'.join(crumbs)))
 
-        return reverse('suave:page', kwargs=kwargs)
+    class Meta:
+        ordering = ['order']
+
+
+class Carousel(models.Model):
+    name = models.CharField(max_length=45, unique=True)
+    images = models.ManyToManyField(Attachment, related_name='carousels',
+        through='CarouselImage')
+
+    def __str__(self):
+        return self.name
+
+    def __unicode__(self):
+        return self.name
+
+
+class CarouselImage(models.Model):
+    image = models.ForeignKey(Attachment)
+    carousel = models.ForeignKey(Carousel)
+    order = models.IntegerField()
 
     class Meta:
         ordering = ['order']
